@@ -1,5 +1,5 @@
 # mautrix-twitter - A Matrix-Twitter DM puppeting bridge
-# Copyright (C) 2020 Tulir Asokan
+# Copyright (C) 2021 Tulir Asokan
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -17,26 +17,26 @@ from typing import (Dict, Tuple, Optional, List, Deque, Set, Any, Union, AsyncGe
                     Awaitable, NamedTuple, TYPE_CHECKING, cast)
 from collections import deque
 from datetime import datetime
-from os import path
 import asyncio
-import html
 
+from yarl import URL
 import magic
 
 from mautwitdm.types import (ConversationType, Conversation, MessageData, Participant, ReactionKey,
-                             MessageAttachmentMedia, TimelineStatus, ReactionCreateEntry,
-                             ReactionDeleteEntry, MessageEntry)
+                             TimelineStatus, ReactionCreateEntry, ReactionDeleteEntry,
+                             MessageEntry, VideoVariant)
 from mautrix.appservice import AppService, IntentAPI
-from mautrix.bridge import BasePortal, NotificationDisabler
+from mautrix.bridge import BasePortal, NotificationDisabler, async_getter_lock
 from mautrix.types import (EventID, MessageEventContent, RoomID, EventType, MessageType,
-                           TextMessageEventContent, MediaMessageEventContent, ImageInfo, VideoInfo,
-                           ContentURI, EncryptedFile)
+                           MediaMessageEventContent, ImageInfo, VideoInfo, ThumbnailInfo,
+                           ContentURI, EncryptedFile, TextMessageEventContent)
 from mautrix.errors import MatrixError, MForbidden
+from mautrix.util.message_send_checkpoint import MessageSendCheckpointStatus
 from mautrix.util.simple_lock import SimpleLock
-from mautrix.util.network_retry import call_with_net_retry
 
 from .db import Portal as DBPortal, Message as DBMessage, Reaction as DBReaction
 from .config import Config
+from .formatter import twitter_to_matrix
 from . import user as u, puppet as p, matrix as m
 
 if TYPE_CHECKING:
@@ -106,7 +106,7 @@ class Portal(DBPortal, BasePortal):
         cls.matrix = bridge.matrix
         cls.az = bridge.az
         cls.loop = bridge.loop
-        cls.bridge = bridge
+        BasePortal.bridge = bridge
         cls.private_chat_portal_meta = cls.config["bridge.private_chat_portal_meta"]
         NotificationDisabler.puppet_cls = p.Puppet
         NotificationDisabler.config_enabled = cls.config["bridge.backfill.disable_notifications"]
@@ -137,15 +137,33 @@ class Portal(DBPortal, BasePortal):
     # endregion
     # region Matrix event handling
 
+    async def _send_error_notice(self, body: str, err: Exception):
+        await self._send_message(
+            self.main_intent,
+            TextMessageEventContent(msgtype=MessageType.NOTICE, body=f"\u26a0 {body}: {err}"),
+        )
+
+    def _status_from_exception(self, e: Exception) -> MessageSendCheckpointStatus:
+        if isinstance(e, NotImplementedError):
+            return MessageSendCheckpointStatus.UNSUPPORTED
+        return MessageSendCheckpointStatus.PERM_FAILURE
+
     async def handle_matrix_message(self, sender: 'u.User', message: MessageEventContent,
                                     event_id: EventID) -> None:
         if not sender.client:
             self.log.debug(f"Ignoring message {event_id} as user is not connected")
             return
-        elif ((message.get(self.bridge.real_user_content_key,
-                           False) and await p.Puppet.get_by_custom_mxid(sender.mxid))):
-            self.log.debug(f"Ignoring puppet-sent message by confirmed puppet user {sender.mxid}")
-            return
+        try:
+            await self._handle_matrix_message(sender, message, event_id)
+        except Exception as e:
+            status = self._status_from_exception(e)
+            sender.send_remote_checkpoint(
+                status, event_id, self.mxid, EventType.ROOM_MESSAGE, message.msgtype, error=e,
+            )
+            await self._send_error_notice("Your message may not have been bridged", e)
+
+    async def _handle_matrix_message(self, sender: 'u.User', message: MessageEventContent,
+                                    event_id: EventID) -> None:
         request_id = str(sender.client.new_request_id())
         self._reqid_dedup.add(request_id)
         text = message.body
@@ -160,62 +178,103 @@ class Portal(DBPortal, BasePortal):
             else:
                 data = await self.main_intent.download_media(message.url)
             mime_type = message.info.mimetype or magic.from_buffer(data, mime=True)
-            # TODO this will throw errors if the mime type is not supported
-            #      those errors should be sent back to the client
             upload_resp = await sender.client.upload(data, mime_type=mime_type)
             media_id = upload_resp.media_id
             text = ""
         resp = await sender.client.conversation(self.twid).send(text, media_id=media_id,
                                                                 request_id=request_id)
+        await self._send_delivery_receipt(event_id)
+        sender.send_remote_checkpoint(
+            status=MessageSendCheckpointStatus.SUCCESS,
+            event_id=event_id,
+            room_id=self.mxid,
+            event_type=EventType.ROOM_MESSAGE,
+            message_type=message.msgtype,
+        )
         resp_msg_id = int(resp.entries[0].message.id)
         self._msgid_dedup.appendleft(resp_msg_id)
         msg = DBMessage(mxid=event_id, mx_room=self.mxid, twid=resp_msg_id, receiver=self.receiver)
         await msg.insert()
         self._reqid_dedup.remove(request_id)
-        await self._send_delivery_receipt(event_id)
         self.log.debug(f"Handled Matrix message {event_id} -> {resp_msg_id}")
 
     async def handle_matrix_reaction(self, sender: 'u.User', event_id: EventID,
                                      reacting_to: EventID, reaction_val: str) -> None:
         try:
-            reaction = ReactionKey.from_emoji(reaction_val)
-        except ValueError:
-            self.log.debug(f"Ignoring unsupported reaction {event_id} with value {reaction_val}")
-            return
+            await self._handle_matrix_reaction(sender, event_id, reacting_to, reaction_val)
+        except Exception as e:
+            status = self._status_from_exception(e)
+            self.log.exception(f"Failed to react to {event_id}")
+            sender.send_remote_checkpoint(
+                status, event_id, self.mxid, EventType.REACTION, error=e
+            )
+            await self._send_error_notice(f"Failed to react to {event_id}", e)
 
+    async def _handle_matrix_reaction(self, sender: 'u.User', event_id: EventID,
+                                     reacting_to: EventID, reaction_val: str) -> None:
+        reaction = ReactionKey.from_emoji(reaction_val)
         message = await DBMessage.get_by_mxid(reacting_to, self.mxid)
         if not message:
-            self.log.debug(f"Ignoring reaction to unknown event {reacting_to}")
-            return
+            raise ValueError(f"Ignoring reaction to unknown event {reacting_to}")
 
-        existing = await DBReaction.get_by_twid(message.twid, message.receiver, sender.twid)
-        if existing and existing.reaction == reaction:
-            return
-
-        dedup_id = (message.twid, sender.twid, reaction)
-        self._reaction_dedup.appendleft(dedup_id)
         async with self._reaction_lock:
+            dedup_id = (message.twid, sender.twid, reaction)
+            self._reaction_dedup.appendleft(dedup_id)
+
+            existing = await DBReaction.get_by_twid(message.twid, message.receiver, sender.twid)
+            if existing and existing.reaction == reaction:
+                return
+
             await sender.client.conversation(self.twid).react(message.twid, reaction)
+            sender.send_remote_checkpoint(
+                MessageSendCheckpointStatus.SUCCESS,
+                event_id,
+                self.mxid,
+                EventType.REACTION,
+            )
+            await self._send_delivery_receipt(event_id)
             await self._upsert_reaction(existing, self.main_intent, event_id, message, sender,
                                         reaction)
-            self.log.trace(f"{sender.mxid} reacted to {message.twid} with {reaction}")
-        await self._send_delivery_receipt(event_id)
+            self.log.debug(f"{sender.mxid} reacted to {message.twid} with {reaction}")
 
     async def handle_matrix_redaction(self, sender: 'u.User', event_id: EventID,
                                       redaction_event_id: EventID) -> None:
-        if not self.mxid:
-            return
+        try:
+            await self._handle_matrix_redaction(sender, event_id)
+        except Exception as e:
+            status = self._status_from_exception(e)
+            sender.send_remote_checkpoint(
+                status, event_id, self.mxid, EventType.ROOM_REDACTION, error=e
+            )
+            await self._send_error_notice(f"Failed to redact {event_id}", e)
+        else:
+            sender.send_remote_checkpoint(
+                MessageSendCheckpointStatus.SUCCESS,
+                redaction_event_id,
+                self.mxid,
+                EventType.ROOM_REDACTION,
+            )
+            await self._send_delivery_receipt(redaction_event_id)
 
-        reaction = await DBReaction.get_by_mxid(event_id, self.mxid)
-        if reaction:
-            try:
-                await reaction.delete()
-                await sender.client.conversation(self.twid).delete_reaction(reaction.tw_msgid,
-                                                                            reaction.reaction)
-                await self._send_delivery_receipt(redaction_event_id)
-                self.log.trace(f"Removed {reaction} after Matrix redaction")
-            except Exception:
-                self.log.exception("Removing reaction failed")
+    async def _handle_matrix_redaction(self, sender: 'u.User', event_id: EventID) -> None:
+        assert self.mxid, "MXID is None"
+
+        async with self._reaction_lock:
+            reaction = await DBReaction.get_by_mxid(event_id, self.mxid)
+            if reaction:
+                try:
+                    await reaction.delete()
+                    await sender.client.conversation(self.twid).delete_reaction(reaction.tw_msgid,
+                                                                                reaction.reaction)
+                except Exception:
+                    self.log.exception("Removing reaction failed")
+                    raise
+                else:
+                    self.log.trace(f"Removed {reaction} after Matrix redaction")
+
+                return
+
+        raise NotImplementedError("Message redactions are not supported.")
 
     async def handle_matrix_leave(self, user: 'u.User') -> None:
         if self.is_direct:
@@ -252,8 +311,7 @@ class Portal(DBPortal, BasePortal):
                 if content:
                     event_id = await self._send_message(intent, content, timestamp=message.time)
             if message.text and not message.text.isspace():
-                message.text = html.unescape(message.text)
-                content = TextMessageEventContent(msgtype=MessageType.TEXT, body=message.text)
+                content = await twitter_to_matrix(message)
                 event_id = await self._send_message(intent, content, timestamp=message.time)
             if event_id:
                 msg = DBMessage(mxid=event_id, mx_room=self.mxid, twid=msg_id,
@@ -262,6 +320,19 @@ class Portal(DBPortal, BasePortal):
                 await self._send_delivery_receipt(event_id)
                 self.log.debug(f"Handled Twitter message {msg_id} -> {event_id}")
 
+    @staticmethod
+    def _is_better_mime(best: VideoVariant, current: VideoVariant) -> bool:
+        order = ["video/mp4"]
+        try:
+            best_quality = order.index(best.content_type)
+        except (IndexError, ValueError):
+            best_quality = -1
+        try:
+            current_quality = order.index(current.content_type)
+        except (IndexError, ValueError):
+            current_quality = -1
+        return current_quality > best_quality
+
     async def _handle_twitter_attachment(self, source: 'u.User', sender: 'p.Puppet',
                                          message: MessageData
                                          ) -> Optional[MediaMessageEventContent]:
@@ -269,10 +340,18 @@ class Portal(DBPortal, BasePortal):
         intent = sender.intent_for(self)
         media = message.attachment.media
         if media:
-            # TODO this doesn't actually work for gifs and videos
-            #      The actual url is in media.video_info.variants[0].url
-            #      Gifs are also videos
-            reuploaded_info = await self._reupload_twitter_media(source, media, intent)
+            reuploaded_info = await self._reupload_twitter_media(source, media.media_url_https,
+                                                                 intent)
+            thumbnail_info = None
+            if media.video_info:
+                thumbnail_info = reuploaded_info
+                best_variant = None
+                for variant in media.video_info.variants:
+                    if ((not best_variant or (variant.bitrate or 0) > (best_variant.bitrate or 0)
+                         or self._is_better_mime(best_variant, variant))):
+                        best_variant = variant
+                reuploaded_info = await self._reupload_twitter_media(source, best_variant.url,
+                                                                     intent)
             content = MediaMessageEventContent(body=reuploaded_info.file_name,
                                                url=reuploaded_info.mxc,
                                                file=reuploaded_info.decryption_info,
@@ -290,6 +369,13 @@ class Portal(DBPortal, BasePortal):
                                          size=reuploaded_info.size,
                                          width=media.original_info.width,
                                          height=media.original_info.height)
+            if thumbnail_info:
+                content.info.thumbnail_url = thumbnail_info.mxc
+                content.info.thumbnail_file = thumbnail_info.decryption_info
+                content.info.thumbnail_info = ThumbnailInfo(mimetype=thumbnail_info.mime_type,
+                                                            size=thumbnail_info.size,
+                                                            width=media.original_info.width,
+                                                            height=media.original_info.height)
             # Remove the attachment link from message.text
             start, end = media.indices
             message.text = message.text[:start] + message.text[end:]
@@ -298,10 +384,10 @@ class Portal(DBPortal, BasePortal):
             pass
         return content
 
-    async def _reupload_twitter_media(self, source: 'u.User', attachment: MessageAttachmentMedia,
-                                      intent: IntentAPI) -> ReuploadedMediaInfo:
-        file_name = path.basename(attachment.media_url_https)
-        data, mime_type = await source.client.download_media(attachment)
+    async def _reupload_twitter_media(self, source: 'u.User', url: str, intent: IntentAPI
+                                      ) -> ReuploadedMediaInfo:
+        file_name = URL(url).name
+        data, mime_type = await source.client.download_media(url)
 
         upload_mime_type = mime_type
         upload_file_name = file_name
@@ -311,8 +397,8 @@ class Portal(DBPortal, BasePortal):
             upload_mime_type = "application/octet-stream"
             upload_file_name = None
 
-        mxc = await call_with_net_retry(intent.upload_media, data, mime_type=upload_mime_type,
-                                        filename=upload_file_name, _action="upload media")
+        mxc = await intent.upload_media(data, mime_type=upload_mime_type,
+                                        filename=upload_file_name)
 
         if decryption_info:
             decryption_info.url = mxc
@@ -351,16 +437,16 @@ class Portal(DBPortal, BasePortal):
             except MForbidden:
                 await self.main_intent.redact(reaction.mx_room, reaction.mxid)
             await reaction.delete()
-            self.log.trace(f"Removed {reaction} after Twitter removal")
+            self.log.debug(f"Removed {reaction} after Twitter removal")
 
     async def handle_twitter_receipt(self, sender: 'p.Puppet', read_up_to: int) -> None:
         message = await DBMessage.get_by_twid(read_up_to, self.receiver)
         if not message:
-            self.log.trace(f"Ignoring read receipt from {sender.twid} "
+            self.log.debug(f"Ignoring read receipt from {sender.twid} "
                            f"up to unknown message {read_up_to}")
             return
 
-        self.log.trace(f"{sender.twid} read messages up to {read_up_to} ({message.mxid})")
+        self.log.debug(f"{sender.twid} read messages up to {read_up_to} ({message.mxid})")
         await sender.intent_for(self).mark_read(message.mx_room, message.mxid)
 
     # endregion
@@ -420,7 +506,7 @@ class Portal(DBPortal, BasePortal):
         for participant in participants:
             twid = int(participant.user_id)
             puppet = await p.Puppet.get_by_twid(twid)
-            await puppet.intent_for(self).ensure_joined(self.mxid)
+            await puppet.intent_for(self).ensure_joined(self.mxid, bot=self.main_intent)
             if participant.last_read_event_id:
                 await self.handle_twitter_receipt(puppet, int(participant.last_read_event_id))
 
@@ -563,14 +649,25 @@ class Portal(DBPortal, BasePortal):
         async with self._create_room_lock:
             return await self._create_matrix_room(source, info)
 
-    async def _update_matrix_room(self, source: 'u.User', info: Conversation) -> None:
-        await self.main_intent.invite_user(self.mxid, source.mxid, check_cache=True)
-        puppet = await p.Puppet.get_by_custom_mxid(source.mxid)
+    def _get_invite_content(self, double_puppet: Optional['p.Puppet']) -> Dict[str, Any]:
+        invite_content = {}
+        if double_puppet:
+            invite_content["fi.mau.will_auto_accept"] = True
+        if self.is_direct:
+            invite_content["is_direct"] = True
+        return invite_content
+
+    async def _add_user(self, user: 'u.User') -> None:
+        puppet = await p.Puppet.get_by_custom_mxid(user.mxid)
+        await self.main_intent.invite_user(self.mxid, user.mxid, check_cache=True,
+                                           extra_content=self._get_invite_content(puppet))
         if puppet:
             did_join = await puppet.intent.ensure_joined(self.mxid)
-            if did_join and self.conv_type == ConversationType.ONE_TO_ONE:
-                await source.update_direct_chats({self.main_intent.mxid: [self.mxid]})
+            if did_join and self.is_direct:
+                await user.update_direct_chats({self.main_intent.mxid: [self.mxid]})
 
+    async def _update_matrix_room(self, source: 'u.User', info: Conversation) -> None:
+        await self._add_user(source)
         await self.update_info(info)
 
         # TODO
@@ -600,7 +697,7 @@ class Portal(DBPortal, BasePortal):
             "state_key": self.bridge_info_state_key,
             "content": self.bridge_info,
         }]
-        invites = [source.mxid]
+        invites = []
         if self.config["bridge.encryption.default"] and self.matrix.e2ee:
             self.encrypted = True
             initial_state.append({
@@ -620,9 +717,16 @@ class Portal(DBPortal, BasePortal):
         # We lock backfill lock here so any messages that come between the room being created
         # and the initial backfill finishing wouldn't be bridged before the backfill messages.
         with self.backfill_lock:
-            self.mxid = await self.main_intent.create_room(name=name, is_direct=self.is_direct,
-                                                           initial_state=initial_state,
-                                                           invitees=invites)
+            creation_content = {}
+            if not self.config["bridge.federate_rooms"]:
+                creation_content["m.federate"] = False
+            self.mxid = await self.main_intent.create_room(
+                name=name,
+                is_direct=self.is_direct,
+                initial_state=initial_state,
+                invitees=invites,
+                creation_content=creation_content,
+            )
             if not self.mxid:
                 raise Exception("Failed to create room: no mxid returned")
 
@@ -636,8 +740,8 @@ class Portal(DBPortal, BasePortal):
             await self.update()
             self.log.debug(f"Matrix room created: {self.mxid}")
             self.by_mxid[self.mxid] = self
-            if not self.is_direct:
-                await self._update_participants(info.participants)
+            await self._add_user(source)
+            await self._update_participants(info.participants)
 
             puppet = await p.Puppet.get_by_custom_mxid(source.mxid)
             if puppet:
@@ -665,6 +769,10 @@ class Portal(DBPortal, BasePortal):
             except Exception:
                 self.log.exception("Failed to backfill new portal")
 
+            # Update participants again after backfill to sync read receipts
+            self._last_participant_update = set()
+            await self._update_participants(info.participants)
+
         return self.mxid
 
     # endregion
@@ -708,6 +816,7 @@ class Portal(DBPortal, BasePortal):
                 yield portal
 
     @classmethod
+    @async_getter_lock
     async def get_by_mxid(cls, mxid: RoomID) -> Optional['Portal']:
         try:
             return cls.by_mxid[mxid]
@@ -722,7 +831,8 @@ class Portal(DBPortal, BasePortal):
         return None
 
     @classmethod
-    async def get_by_twid(cls, twid: str, receiver: int = 0,
+    @async_getter_lock
+    async def get_by_twid(cls, twid: str, *, receiver: int = 0,
                           conv_type: Optional[ConversationType] = None) -> Optional['Portal']:
         if conv_type == ConversationType.GROUP_DM and receiver != 0:
             receiver = 0
