@@ -15,7 +15,6 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 from typing import (Dict, Optional, AsyncIterable, Awaitable, AsyncGenerator, Union, List,
                     TYPE_CHECKING, cast)
-from collections import defaultdict
 import asyncio
 import logging
 
@@ -23,9 +22,10 @@ from mautwitdm import TwitterAPI
 from mautwitdm.poller import PollingStopped, PollingStarted, PollingErrored, PollingErrorResolved
 from mautwitdm.types import (MessageEntry, ReactionCreateEntry, ReactionDeleteEntry, Conversation,
                              User as TwitterUser, ConversationReadEntry)
-from mautrix.bridge import BaseUser
+from mautwitdm.errors import TwitterAuthError, TwitterError
+from mautrix.bridge import BaseUser, async_getter_lock
+from mautrix.util.bridge_state import BridgeState, BridgeStateEvent
 from mautrix.types import UserID, RoomID, EventID, TextMessageEventContent, MessageType
-from mautrix.appservice import AppService
 from mautrix.util.opt_prometheus import Summary, Gauge, async_time
 
 from .db import User as DBUser, Portal as DBPortal
@@ -44,39 +44,43 @@ METRIC_RECEIPT = Summary("bridge_on_receipt", "calls to handle_receipt")
 METRIC_LOGGED_IN = Gauge("bridge_logged_in", "Users logged into the bridge")
 METRIC_CONNECTED = Gauge("bridge_connected", "Bridged users connected to Twitter")
 
+BridgeState.human_readable_errors.update({
+    "logged-out": "You're not logged into Twitter",
+    "twitter-connection-failed": "Failed to connect to Twitter",
+    "twitter-not-connected": None,
+    "twitter-connection-error": "An error occurred while polling Twitter for new messages",
+})
+
 
 class User(DBUser, BaseUser):
     by_mxid: Dict[UserID, 'User'] = {}
     by_twid: Dict[int, 'User'] = {}
     config: Config
-    az: AppService
-    loop: asyncio.AbstractEventLoop
 
     client: Optional[TwitterAPI]
 
-    is_admin: bool
     permission_level: str
     username: Optional[str]
 
     _notice_room_lock: asyncio.Lock
     _notice_send_lock: asyncio.Lock
     _is_logged_in: Optional[bool]
+    _connected: bool
 
     def __init__(self, mxid: UserID, twid: Optional[int] = None, auth_token: Optional[str] = None,
                  csrf_token: Optional[str] = None, poll_cursor: Optional[str] = None,
                  notice_room: Optional[RoomID] = None) -> None:
         super().__init__(mxid=mxid, twid=twid, auth_token=auth_token, csrf_token=csrf_token,
                          poll_cursor=poll_cursor, notice_room=notice_room)
+        BaseUser.__init__(self)
         self._notice_room_lock = asyncio.Lock()
         self._notice_send_lock = asyncio.Lock()
         perms = self.config.get_permissions(mxid)
         self.is_whitelisted, self.is_admin, self.permission_level = perms
-        self.log = self.log.getChild(self.mxid)
         self.client = None
         self.username = None
-        self.dm_update_lock = asyncio.Lock()
-        self._metric_value = defaultdict(lambda: False)
         self._is_logged_in = None
+        self._connected = False
 
     @classmethod
     def init_cls(cls, bridge: 'TwitterBridge') -> AsyncIterable[Awaitable[None]]:
@@ -109,8 +113,20 @@ class User(DBUser, BaseUser):
     async def try_connect(self) -> None:
         try:
             await self.connect()
-        except Exception:
+        except TwitterAuthError as e:
+            self.log.exception("Auth error while connecting to Twitter")
+            await self.push_bridge_state(BridgeStateEvent.BAD_CREDENTIALS,
+                                         error="twitter-auth-error",
+                                         message=e.message)
+        except TwitterError as e:
             self.log.exception("Error while connecting to Twitter")
+            await self.push_bridge_state(BridgeStateEvent.UNKNOWN_ERROR,
+                                         error="twitter-unknown-error",
+                                         message=e.message)
+        except Exception:
+            self.log.exception("Unknown exception while connecting to Twitter")
+            await self.push_bridge_state(BridgeStateEvent.UNKNOWN_ERROR,
+                                         error="twitter-connection-failed")
 
     async def connect(self, auth_token: Optional[str] = None, csrf_token: Optional[str] = None
                       ) -> None:
@@ -147,14 +163,35 @@ class User(DBUser, BaseUser):
             self.log.debug("Poll cursor set, starting polling right away (not initial syncing)")
             self.client.start_polling()
         else:
-            self.loop.create_task(self._try_initial_sync())
-        self.loop.create_task(self._try_sync_puppet(user_info))
+            asyncio.create_task(self._try_initial_sync())
+        asyncio.create_task(self._try_sync_puppet(user_info))
 
-    async def on_connect(self, evt: PollingStarted) -> None:
+    async def fill_bridge_state(self, state: BridgeState) -> None:
+        await super().fill_bridge_state(state)
+        if self.twid:
+            state.remote_id = str(self.twid)
+            puppet = await pu.Puppet.get_by_twid(self.twid)
+            state.remote_name = puppet.name
+
+    async def get_bridge_states(self) -> List[BridgeState]:
+        if not self.twid:
+            return []
+        state = BridgeState(state_event=BridgeStateEvent.UNKNOWN_ERROR)
+        if self._connected:
+            state.state_event = BridgeStateEvent.CONNECTED
+        return [state]
+
+    async def on_connect(self, evt: Union[PollingStarted, PollingErrorResolved]) -> None:
         self._track_metric(METRIC_CONNECTED, True)
+        self._connected = True
+        await self.push_bridge_state(BridgeStateEvent.CONNECTED)
 
     async def on_disconnect(self, evt: Union[PollingStopped, PollingErrored]) -> None:
         self._track_metric(METRIC_CONNECTED, False)
+        self._connected = False
+        if isinstance(evt, PollingStopped):
+            await self.push_bridge_state(BridgeStateEvent.UNKNOWN_ERROR,
+                                         error="twitter-not-connected")
 
     # TODO this stuff could probably be moved to mautrix-python
     async def get_notice_room(self) -> RoomID:
@@ -164,14 +201,25 @@ class User(DBUser, BaseUser):
                 # don't make a new room
                 if self.notice_room:
                     return self.notice_room
+                creation_content = {}
+                if not self.config["bridge.federate_rooms"]:
+                    creation_content["m.federate"] = False
                 self.notice_room = await self.az.intent.create_room(
-                    is_direct=True, invitees=[self.mxid],
-                    topic="Twitter DM bridge notices")
+                    is_direct=True,
+                    invitees=[self.mxid],
+                    topic="Twitter DM bridge notices",
+                    creation_content=creation_content,
+                )
                 await self.update()
         return self.notice_room
 
     async def send_bridge_notice(self, text: str, edit: Optional[EventID] = None,
+                                 state_event: Optional[BridgeStateEvent] = None,
                                  important: bool = False) -> Optional[EventID]:
+        if state_event:
+            await self.push_bridge_state(state_event, message=text)
+        if self.config["bridge.disable_bridge_notices"]:
+            return None
         event_id = None
         try:
             self.log.debug("Sending bridge notice: %s", text)
@@ -189,14 +237,22 @@ class User(DBUser, BaseUser):
     async def on_error(self, evt: PollingErrored) -> None:
         if evt.fatal:
             await self.send_bridge_notice(f"Fatal error while polling Twitter: {evt.error}",
+                                          state_event=BridgeStateEvent.UNKNOWN_ERROR,
                                           important=evt.fatal)
         elif evt.count == 1 and self.config["bridge.temporary_disconnect_notices"]:
-            await self.send_bridge_notice(f"Error while polling Twitter: {evt.error}  \n"
-                                          f"The bridge will keep retrying.")
+            await self.send_bridge_notice(
+                f"Error while polling Twitter: {evt.error}\nThe bridge will keep retrying.",
+                state_event=BridgeStateEvent.TRANSIENT_DISCONNECT,
+            )
+        else:
+            state_event = (BridgeStateEvent.TRANSIENT_DISCONNECT if evt.count < 5
+                           else BridgeStateEvent.UNKNOWN_ERROR)
+            await self.push_bridge_state(state_event, f"Error while polling Twitter: {evt.error}")
 
     async def on_error_resolved(self, evt: PollingErrorResolved) -> None:
         if self.config["bridge.temporary_disconnect_notices"]:
-            await self.send_bridge_notice("Twitter polling error resolved")
+            await self.send_bridge_notice("Twitter polling error resolved",
+                                          state_event=BridgeStateEvent.CONNECTED)
 
     async def _try_sync_puppet(self, user_info: TwitterUser) -> None:
         puppet = await pu.Puppet.get_by_twid(self.twid)
@@ -227,6 +283,7 @@ class User(DBUser, BaseUser):
         }
 
     async def sync(self) -> None:
+        await self.push_bridge_state(BridgeStateEvent.BACKFILLING)
         resp = await self.client.inbox_initial_state(set_poll_cursor=False)
         if not self.poll_cursor:
             self.poll_cursor = resp.cursor
@@ -237,8 +294,12 @@ class User(DBUser, BaseUser):
             limit = len(conversations)
         for user in resp.users.values():
             await self.handle_user_update(user)
-        for i, conversation in enumerate(conversations):
-            await self.handle_conversation_update(conversation, create_portal=i < limit)
+        index = 0
+        for conversation in conversations:
+            create_portal = index < limit and conversation.trusted
+            if create_portal:
+                index += 1
+            await self.handle_conversation_update(conversation, create_portal=create_portal)
         await self.update_direct_chats()
 
     async def get_info(self) -> TwitterUser:
@@ -278,7 +339,8 @@ class User(DBUser, BaseUser):
     @async_time(METRIC_CONVERSATION_UPDATE)
     async def handle_conversation_update(self, evt: Conversation, create_portal: bool = False
                                          ) -> None:
-        portal = await po.Portal.get_by_twid(evt.conversation_id, self.twid, conv_type=evt.type)
+        portal = await po.Portal.get_by_twid(evt.conversation_id, receiver=self.twid,
+                                             conv_type=evt.type)
         if not portal.mxid:
             if create_portal:
                 await portal.create_matrix_room(self, evt)
@@ -294,7 +356,7 @@ class User(DBUser, BaseUser):
 
     @async_time(METRIC_MESSAGE)
     async def handle_message(self, evt: MessageEntry) -> None:
-        portal = await po.Portal.get_by_twid(evt.conversation_id, self.twid,
+        portal = await po.Portal.get_by_twid(evt.conversation_id, receiver=self.twid,
                                              conv_type=evt.conversation.type)
         if not portal.mxid:
             await portal.create_matrix_room(self, evt.conversation)
@@ -303,7 +365,7 @@ class User(DBUser, BaseUser):
 
     @async_time(METRIC_REACTION)
     async def handle_reaction(self, evt: Union[ReactionCreateEntry, ReactionDeleteEntry]) -> None:
-        portal = await po.Portal.get_by_twid(evt.conversation_id, self.twid,
+        portal = await po.Portal.get_by_twid(evt.conversation_id, receiver=self.twid,
                                              conv_type=evt.conversation.type)
         if not portal.mxid:
             self.log.debug(f"Ignoring reaction in conversation {evt.conversation_id} with no room")
@@ -318,7 +380,7 @@ class User(DBUser, BaseUser):
 
     @async_time(METRIC_RECEIPT)
     async def handle_receipt(self, evt: ConversationReadEntry) -> None:
-        portal = await po.Portal.get_by_twid(evt.conversation_id, self.twid,
+        portal = await po.Portal.get_by_twid(evt.conversation_id, receiver=self.twid,
                                              conv_type=evt.conversation.type)
         if not portal.mxid:
             return
@@ -334,7 +396,8 @@ class User(DBUser, BaseUser):
             self.by_twid[self.twid] = self
 
     @classmethod
-    async def get_by_mxid(cls, mxid: UserID, create: bool = True) -> Optional['User']:
+    @async_getter_lock
+    async def get_by_mxid(cls, mxid: UserID, *, create: bool = True) -> Optional['User']:
         # Never allow ghosts to be users
         if pu.Puppet.get_id_from_mxid(mxid):
             return None
@@ -357,6 +420,7 @@ class User(DBUser, BaseUser):
         return None
 
     @classmethod
+    @async_getter_lock
     async def get_by_twid(cls, twid: int) -> Optional['User']:
         try:
             return cls.by_twid[twid]
