@@ -6,13 +6,14 @@
 from typing import Dict, Optional, Type, TypeVar, List, Callable, Awaitable, Any, Union
 import logging
 import asyncio
+import time
 
 from attr import dataclass
 from aiohttp import ClientSession
 from yarl import URL
 
 from .types import PollResponse, InitialStateResponse
-from .errors import check_error
+from .errors import check_error, RateLimitError
 from .conversation import Conversation
 from .dispatcher import TwitterDispatcher
 
@@ -122,7 +123,18 @@ class TwitterPoller(TwitterDispatcher):
         })
         async with self.http.get(url, headers=self.headers) as resp:
             data = await check_error(resp)
-            response = PollResponse.deserialize(data["user_events"])
+            try:
+                user_events = data["user_events"]
+            except KeyError:
+                try:
+                    inbox_initial_state = data["inbox_initial_state"]
+                except KeyError:
+                    self.log.warning("Got unknown poll response: %s", data)
+                    raise
+                else:
+                    response = InitialStateResponse.deserialize(inbox_initial_state)
+            else:
+                response = PollResponse.deserialize(user_events)
             self.poll_cursor = response.cursor
             return response
 
@@ -154,8 +166,21 @@ class TwitterPoller(TwitterDispatcher):
             if not entry:
                 continue
             for entry_type in entry.all_types:
-                entry_type.conversation = resp.conversations[entry_type.conversation_id]
-                await self.dispatch(entry_type)
+                try:
+                    entry_type.conversation = resp.conversations[entry_type.conversation_id]
+                except KeyError:
+                    msg = ("Poll response didn't contain conversation info "
+                           f"for {entry_type.conversation_id} "
+                           f"(entry type {type(entry_type)}, ID {entry_type.id}")
+                    if entry_type.conversation:
+                        self.log.debug(f"{msg}, but the entry had its own conversation info")
+                        await self.dispatch(entry_type)
+                    else:
+                        # TODO we could still dispatch the event and guess the conversation type
+                        #      in event handlers if necessary.
+                        self.log.warning(msg)
+                else:
+                    await self.dispatch(entry_type)
 
     async def _poll_forever(self) -> None:
         if not self.poll_cursor:
@@ -168,6 +193,15 @@ class TwitterPoller(TwitterDispatcher):
         while True:
             try:
                 resp = await self._poll_once()
+            except RateLimitError as e:
+                sleep = e.reset - int(time.time())
+                self.log.warning(f"Got rate limit until {e.reset} while polling, "
+                                 f"waiting {sleep} seconds")
+                if self.poll_sleep < 8:
+                    self.poll_sleep += 1
+                    self.log.debug(f"Increased poll sleep to {self.poll_sleep}")
+                await asyncio.sleep(sleep)
+                continue
             except Exception as e:
                 if errors > self.max_poll_errors > 0:
                     self.log.debug(f"Error count ({errors}) exceeded maximum, "
@@ -203,7 +237,7 @@ class TwitterPoller(TwitterDispatcher):
         """
         self.stop_polling()
         self.log.debug("Starting poll task")
-        self._poll_task = self.loop.create_task(self.poll_forever())
+        self._poll_task = asyncio.create_task(self.poll_forever())
         return self._poll_task
 
     def stop_polling(self) -> None:
