@@ -1,9 +1,9 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import List, Tuple
+from typing import List, Tuple, Union
 
 from attr import dataclass
 
@@ -11,7 +11,7 @@ from mautrix.types import SerializableAttrs
 
 
 @dataclass
-class MessageEntityURL(SerializableAttrs['MessageEntityURL']):
+class MessageEntityURL(SerializableAttrs):
     url: str
     expanded_url: str
     display_url: str
@@ -19,7 +19,7 @@ class MessageEntityURL(SerializableAttrs['MessageEntityURL']):
 
 
 @dataclass
-class MessageEntityUserMention(SerializableAttrs['MessageEntityUserMention']):
+class MessageEntityUserMention(SerializableAttrs):
     screen_name: str
     name: str
     id: int
@@ -28,14 +28,19 @@ class MessageEntityUserMention(SerializableAttrs['MessageEntityUserMention']):
 
 
 @dataclass
-class MessageEntitySimple(SerializableAttrs['MessageEntitySimple']):
+class MessageEntitySimple(SerializableAttrs):
     text: str
     indices: Tuple[int, int]
 
 
 @dataclass
-class MessageEntities(SerializableAttrs['MessageEntities']):
+class MessageEntities(SerializableAttrs):
     hashtags: List[MessageEntitySimple]
     symbols: List[MessageEntitySimple]
     user_mentions: List[MessageEntityUserMention]
     urls: List[MessageEntityURL]
+
+    @property
+    def all(self) -> List[Union[MessageEntitySimple, MessageEntityUserMention, MessageEntityURL]]:
+        entities = self.hashtags + self.symbols + self.user_mentions + self.urls
+        return sorted(entities, key=lambda entity: entity.indices[0])
